function g = mlpardKernGradient(kern, x, varargin)

% MLPARDKERNGRADIENT Gradient of multi-layer perceptron ARD kernel's parameters.
%
% g = mlpardKernGradient(kern, x, varargin)
%

% Copyright (c) 2006 Neil D. Lawrence and Nathaniel J. King
% mlpardKernGradient.m version 1.4


g = zeros(1, kern.nParams);
if nargin < 4
  [k, innerProd, arg, denom, numer, vecDenom] ...
      = mlpardKernCompute(kern, x);
else
  [k, innerProd, arg, denom, numer, vecDenom1, vecDenom2] ...
      = mlpardKernCompute(kern, x, varargin{1});
end

denom3 = denom.*denom.*denom;
base = kern.variance./sqrt(1-arg.*arg);
baseCovGrad = base.*varargin{end};
if nargin < 4
  vec = diag(innerProd);
  g(1) = sum(sum((innerProd./denom ...
                  -.5*numer./denom3 ...
                  .*((kern.weightVariance.*vec+kern.biasVariance+1)*vec' ...
                     + vec*(kern.weightVariance.*vec+kern.biasVariance+1)')) ...
                 .*baseCovGrad));
  
  g(2) = sum(sum((1./denom ...
                  -.5*numer./denom3 ...
                  .*(repmat(vec, 1, size(vec, 1))*kern.weightVariance...
                     + 2*kern.biasVariance + 2 ...
                     +repmat(vec', size(vec, 1), 1)*kern.weightVariance))...
                 .*baseCovGrad));
else
  scales = sparse(diag(sqrt(kern.inputScales)));
  xt = x*scales;
  vec1 = sum(xt.*xt, 2);
  xt=varargin{1}*scales;
  vec2 = sum(xt.*xt, 2);
  g(1) = sum(sum((innerProd./denom ...
                  -.5*numer./denom3...
                  .*((kern.weightVariance.*vec1+kern.biasVariance+1)*vec2' ...
                     + vec1*(kern.weightVariance.*vec2+kern.biasVariance+1)')).*baseCovGrad));
  g(2) = sum(sum((1./denom ...
                  -.5*numer./denom3 ...
                  .*(repmat(vec1, 1, size(vec2, 1))*kern.weightVariance...
                     + 2*kern.biasVariance + 2 ...
                     +repmat(vec2', size(vec1, 1), 1)* ...
                     kern.weightVariance)).*baseCovGrad));
end

g(3) = sum(sum(k.*varargin{end}))/kern.variance;
if nargin < 4
  for j = 1:kern.inputDimension
    x2 = x(:, j).*x(:, j);
    g(3+j) = sum(sum(((x(:, j)*x(:, j)')./denom ...
                      -.5*numer./denom3 ...
                      .*(x2*vecDenom' + vecDenom*x2'))...
                     .*baseCovGrad))*kern.weightVariance;
  end
else
  for j = 1:kern.inputDimension
    x21 = x(:, j).*x(:, j);
    x22 = varargin{1}(:, j).*varargin{1}(:, j);
    g(3+j) = sum(sum(((x(:, j)*varargin{1}(:, j)')./denom ...
                      -.5*numer./denom3 ...
                      .*(x21*vecDenom2' + vecDenom1*x22'))...
                     .*baseCovGrad))*kern.weightVariance;
  end
end
    

